@@ -17,7 +17,7 @@
 		"exportFileData": false,
 		"useJournalAbbreviation": false
 	},
-	"lastUpdated": "2019-12-15 14:26:00"
+	"lastUpdated": "2022-10-12 19:26:00"
 }
 
 /*
@@ -484,6 +484,12 @@ function encodeFilePathComponent(value) {
 	return value.replace(encodeFilePathRE, "\\$&");
 }
 
+// We strip out {} in general, because \{ and \} break BibLaTeX
+function cleanFilePath(str) {
+	if (!str) return '';
+	return str.replace(/(?:\s*[{}]+)+\s*/g, ' ');
+}
+
 function doExport() {
 	// Zotero.write("% biblatex export generated by Zotero "+Zotero.Utilities.getVersion());
 	// to make sure the BOM gets ignored
@@ -848,16 +854,21 @@ function doExport() {
 
 			for (let i = 0; i < item.attachments.length; i++) {
 				var attachment = item.attachments[i];
+				var title = cleanFilePath(attachment.title),
+					path = null;
+
 				if (Zotero.getOption("exportFileData") && attachment.saveFile) {
-					attachment.saveFile(attachment.defaultPath, true);
-					attachmentString += ";" + encodeFilePathComponent(attachment.title) + ":"
-						+ encodeFilePathComponent(attachment.defaultPath) + ":"
-						+ encodeFilePathComponent(attachment.mimeType);
+					path = cleanFilePath(attachment.defaultPath);
+					attachment.saveFile(path, true);
 				}
 				else if (attachment.localPath) {
-					attachmentString += ";" + encodeFilePathComponent(attachment.title) + ":"
-						+ encodeFilePathComponent(attachment.localPath) + ":"
-						+ encodeFilePathComponent(attachment.mimeType);
+					path = cleanFilePath(attachment.localPath);
+				}
+
+				if (path) {
+					attachmentString += ";" + encodeFilePathComponent(title)
+						+ ":" + encodeFilePathComponent(path)
+						+ ":" + encodeFilePathComponent(attachment.mimeType);
 				}
 			}
 
@@ -868,4 +879,6 @@ function doExport() {
 
 		Zotero.write(",\n}");
 	}
+
+	Zotero.write("\n");
 }
