@@ -25,29 +25,29 @@
 
 
 /*
-    * Base function for retrieving ids and names of static types stored in the DB
-    * (e.g. creatorType, fileType, charset, itemType)
-    *
-    * Extend using the following code within a child constructor:
-    *
-    * 	Zotero.CachedTypes.apply(this, arguments);
-    *  this.constructor.prototype = new Zotero.CachedTypes();
-    *
-    * And the following properties:
-    *
-    *	this._typeDesc = '';
-    *	this._typeDescPlural = '';
-    *	this._idCol = '';
-    *	this._nameCol = '';
-    *	this._table = '';
-    *
-    * Optional properties:
-    *
-    *  this._allowAdd: Allow new types to be added via .add(name)
-    *	this._ignoreCase: Ignore case when looking for types, and add new types as lowercase
-    *
-    * And add .init() to zotero.js
-    */
+ * Base function for retrieving ids and names of static types stored in the DB
+ * (e.g. creatorType, fileType, charset, itemType)
+ *
+ * Extend using the following code within a child constructor:
+ *
+ * 	Zotero.CachedTypes.apply(this, arguments);
+ *  this.constructor.prototype = new Zotero.CachedTypes();
+ *
+ * And the following properties:
+ *
+ *	this._typeDesc = '';
+ *	this._typeDescPlural = '';
+ *	this._idCol = '';
+ *	this._nameCol = '';
+ *	this._table = '';
+ *
+ * Optional properties:
+ *
+ *  this._allowAdd: Allow new types to be added via .add(name)
+ *	this._ignoreCase: Ignore case when looking for types, and add new types as lowercase
+ *
+ * And add .init() to zotero.js
+ */
 Zotero.CachedTypes = function () {
   this._types = null;
   this._typesArray = null;
@@ -133,11 +133,11 @@ Zotero.CachedTypes = function () {
 
 
   /**
-      * Add a new type to the data and return its id. If the type already exists, return its id.
-      *
-      * @param {String} name - Type name to add
-      * @return {Integer|False} - The type id (new or existing), or false if invalid type name
-      */
+   * Add a new type to the data and return its id. If the type already exists, return its id.
+   *
+   * @param {String} name - Type name to add
+   * @return {Integer|False} - The type id (new or existing), or false if invalid type name
+   */
   this.add = Zotero.Promise.coroutine(function* (name) {
     if (!this._allowAdd) {
       throw new Error("New " + this._typeDescPlural + " cannot be added");
@@ -182,8 +182,8 @@ Zotero.CachedTypes = function () {
 
 
   /**
-      * @return {Promise}
-      */
+   * @return {Promise}
+   */
   this._getTypesFromDB = function (where, params) {
     return Zotero.DB.queryAsync(
     'SELECT ' + this._idCol + ' AS id, ' +
@@ -259,7 +259,7 @@ Zotero.CreatorTypes = new function () {
         if (a.primaryField != b.primaryField) return b.primaryField - a.primaryField;
         return Zotero.localeCompare(a.localizedName, b.localizedName);
       });
-      _creatorTypesByItemType[itemTypeID].forEach(x => {
+      _creatorTypesByItemType[itemTypeID].forEach((x) => {
         delete x.primaryField;
         delete x.localizedName;
       });
@@ -352,12 +352,10 @@ Zotero.ItemTypes = new function () {
   this._hasCustom = true;
 
   var _primaryTypeNames = ['book', 'bookSection', 'journalArticle', 'newspaperArticle', 'document'];
-  var _primaryTypes;
-  var _secondaryTypes;
+  // Item types hidden from New Item menu
+  var _hiddenTypeNames = ['webpage', 'attachment', 'note', 'annotation'];
   var _hiddenTypes;
 
-  var _numPrimary = 5;
-
   var _customImages = {};
   var _customLabels = {};
 
@@ -365,18 +363,10 @@ Zotero.ItemTypes = new function () {
   this.init = Zotero.Promise.coroutine(function* () {
     yield this.constructor.prototype.init.apply(this);
 
-    _primaryTypes = yield this._getTypesFromDB(
-    `WHERE typeName IN ('${_primaryTypeNames.join("', '")}')`);
-
-
-    // Secondary types
-    _secondaryTypes = yield this._getTypesFromDB(
-    `WHERE display != 0 AND display NOT IN ('${_primaryTypeNames.join("', '")}')` +
-    " AND name != 'webpage'");
-
-
     // Hidden types
-    _hiddenTypes = yield this._getTypesFromDB('WHERE display=0');
+    _hiddenTypes = yield this._getTypesFromDB(
+    `WHERE typeName IN ('${_hiddenTypeNames.join("', '")}')`);
+
 
     // Custom labels and icons
     var sql = "SELECT customItemTypeID AS id, label, icon FROM customItemTypes";
@@ -391,44 +381,41 @@ Zotero.ItemTypes = new function () {
 
 
   this.getPrimaryTypes = function () {
-    if (!_primaryTypes) {
-      throw new Zotero.Exception.UnloadedDataException("Primary item type data not yet loaded");
-    }
+    var names = _primaryTypeNames.concat();
 
     var mru = Zotero.Prefs.get('newItemTypeMRU');
     if (mru && mru.length) {
       // Get types from the MRU list
       mru = new Set(
       mru.split(',').
-      slice(0, _numPrimary).
-      map(name => this.getName(name))
-      // Ignore 'webpage' item type
-      .filter(name => name && name != 'webpage'));
+      slice(0, _primaryTypeNames.length).
+      map((name) => this.getName(name))
+      // Ignore hidden item types and 'webpage'
+      .filter((name) => name && !_hiddenTypeNames.concat('webpage').includes(name)));
 
 
       // Add types from defaults until we reach our limit
-      for (let i = 0; i < _primaryTypes.length && mru.size < _numPrimary; i++) {
-        mru.add(_primaryTypes[i].name);
+      for (let name of _primaryTypeNames) {
+        if (mru.size >= _primaryTypeNames.length) break;
+        mru.add(name);
       }
 
-      return Array.from(mru).map(name => ({ id: this.getID(name), name }));
+      names = Array.from(mru);
     }
 
-    return _primaryTypes;
+    return names.map((name) => ({ id: this.getID(name), name }));
   };
 
   this.getSecondaryTypes = function () {
-    if (!_secondaryTypes) {
-      throw new Zotero.Exception.UnloadedDataException("Secondary item type data not yet loaded");
-    }
-    return _secondaryTypes;
+    var namesToRemove = new Set(this.getPrimaryTypes().map((x) => x.name).concat(_hiddenTypeNames));
+    return this._typesArray.filter((x) => !namesToRemove.has(x.name));
   };
 
   this.getHiddenTypes = function () {
     if (!_hiddenTypes) {
       throw new Zotero.Exception.UnloadedDataException("Hidden item type data not yet loaded");
     }
-    return _hiddenTypes;
+    return _hiddenTypes.concat();
   };
 
   this.getLocalizedString = function (idOrName) {
@@ -498,6 +485,7 @@ Zotero.ItemTypes = new function () {
       case 'newspaperArticle':
       case 'note':
       case 'patent':
+      case 'preprint':
       case 'presentation':
       case 'report':
       case 'statute':
@@ -530,8 +518,8 @@ Zotero.FileTypes = new function () {
   this._table = 'fileTypes';
 
   /**
-                              * @return {Promise<Integer>} fileTypeID
-                              */
+   * @return {Promise<Integer>} fileTypeID
+   */
   this.getIDFromMIMEType = function (mimeType) {
     var sql = "SELECT fileTypeID FROM fileTypeMIMETypes " +
     "WHERE ? LIKE mimeType || '%'";
@@ -669,7 +657,7 @@ Zotero.CharacterSets = new function () {
   let charsetMap = {};
   for (let canonical in charsetList) {
     charsetMap[canonical.toLowerCase()] = canonical;
-    charsetList[canonical].forEach(c => charsetMap[c.toLowerCase()] = canonical);
+    charsetList[canonical].forEach((c) => charsetMap[c.toLowerCase()] = canonical);
 
     if (!compatibilityNames[canonical]) {
       compatibilityNames[canonical] = canonical;
