@@ -1,7 +1,10 @@
 Zotero.Locale = {
   /**
-                   * Keep this up to date with chrome.manifest
-                   */
+   * Keep this up to date with zotero-standalone-build:assets/multilocale.txt,
+   * zotero-build/dictionaries/build-dictionaries, and possibly chrome.manifest
+   *
+   * Names from https://addons.mozilla.org/en-US/firefox/language-tools/
+   */
   availableLocales: Object.freeze({
     'ar': 'عربي',
     'bg-BG': 'Български',
@@ -10,11 +13,15 @@ Zotero.Locale = {
     'cs-CZ': 'Čeština',
     'da-DK': 'Dansk',
     'de': 'Deutsch',
+    'el-GR': 'Ελληνικά',
+    'en-AU': 'English (Australian)',
     'en-CA': 'English (Canada)',
     'en-US': 'English',
     'en-GB': 'English (UK)',
+    'en-NZ': 'English (New Zealand)',
     'es-ES': 'Español',
     'et-EE': 'Eesti keel',
+    'eu-ES': 'Euskara',
     'fa': 'فارسی',
     'fi-FI': 'suomi',
     'fr-FR': 'Français',
