@@ -50,9 +50,9 @@ Zotero.RecognizePDF = new function () {
   this.recognizeStub = null;
 
   /**
-                              * Triggers queue processing and returns when all items in the queue are processed
-                              * @return {Promise}
-                              */
+   * Triggers queue processing and returns when all items in the queue are processed
+   * @return {Promise}
+   */
   async function _processQueue() {
     await Zotero.Schema.schemaUpdatePromise;
 
@@ -103,9 +103,9 @@ Zotero.RecognizePDF = new function () {
 
 
   /**
-     * Adds items to the queue and triggers processing
-     * @param {Zotero.Item[]} items
-     */
+   * Adds items to the queue and triggers processing
+   * @param {Zotero.Item[]} items
+   */
   this.recognizeItems = async function (items) {
     for (let item of items) {
       if (
@@ -123,10 +123,10 @@ Zotero.RecognizePDF = new function () {
 
 
   /**
-      * Checks whether a given PDF could theoretically be recognized
-      * @param {Zotero.Item} item
-      * @return {Boolean} True if the PDF can be recognized, false if it cannot be
-      */
+   * Checks whether a given PDF could theoretically be recognized
+   * @param {Zotero.Item} item
+   * @return {Boolean} True if the PDF can be recognized, false if it cannot be
+   */
   this.canRecognize = function (item) {
     return item.attachmentContentType &&
     item.attachmentContentType === 'application/pdf' &&
@@ -137,7 +137,7 @@ Zotero.RecognizePDF = new function () {
   this.autoRecognizeItems = async function (items) {
     if (!Zotero.Prefs.get('autoRecognizeFiles')) return;
 
-    var pdfs = items.filter(item => {
+    var pdfs = items.filter((item) => {
       return item &&
       item.isFileAttachment() &&
       item.attachmentContentType == 'application/pdf';
@@ -243,10 +243,10 @@ Zotero.RecognizePDF = new function () {
 
 
   /**
-      * Processes the item and places it as a children of the new item
-      * @param itemID
-      * @return {Promise}
-      */
+   * Processes the item and places it as a children of the new item
+   * @param itemID
+   * @return {Promise}
+   */
   async function _processItem(attachment) {
     // Make sure the attachment still doesn't have a parent
     if (attachment.parentItemID) {
@@ -302,11 +302,9 @@ Zotero.RecognizePDF = new function () {
     }
 
     try {
-      zp = Zotero.getActiveZoteroPane();
-      if (zp) {
-        if (selectParent) {
-          await zp.selectItem(parentItem.id);
-        }
+      let win = Zotero.getMainWindow();
+      if (selectParent && win && win.Zotero_Tabs.selectedID == 'zotero-pane') {
+        await win.ZoteroPane.selectItem(parentItem.id);
       }
     }
     catch (e) {
@@ -325,11 +323,11 @@ Zotero.RecognizePDF = new function () {
   }
 
   /**
-     * Get json from a PDF
-     * @param {String} filePath PDF file path
-     * @param {Number} pages Number of pages to extract
-     * @return {Promise}
-     */
+   * Get json from a PDF
+   * @param {String} filePath PDF file path
+   * @param {Number} pages Number of pages to extract
+   * @return {Promise}
+   */
   async function extractJSON(filePath, pages) {
     let cacheFile = Zotero.getTempDirectory();
     cacheFile.append("recognizePDFcache.txt");
@@ -340,7 +338,7 @@ Zotero.RecognizePDF = new function () {
     let { exec, args } = Zotero.Fulltext.getPDFConverterExecAndArgs();
     args.push('-json', '-l', pages, filePath, cacheFile.path);
 
-    Zotero.debug("RecognizePDF: Running " + exec.path + " " + args.map(arg => "'" + arg + "'").join(" "));
+    Zotero.debug("RecognizePDF: Running " + exec.path + " " + args.map((arg) => "'" + arg + "'").join(" "));
 
     try {
       await Zotero.Utilities.Internal.exec(exec, args);
@@ -363,9 +361,9 @@ Zotero.RecognizePDF = new function () {
   }
 
   /**
-     * Attach appropriate handlers to a Zotero.Translate instance and begin translation
-     * @return {Promise}
-     */
+   * Attach appropriate handlers to a Zotero.Translate instance and begin translation
+   * @return {Promise}
+   */
   async function _promiseTranslate(translate, libraryID) {
     translate.setHandler('select', function (translate, items, callback) {
       for (let i in items) {
@@ -405,10 +403,10 @@ Zotero.RecognizePDF = new function () {
   }
 
   /**
-     * Retrieves metadata for a PDF and saves it as an item
-     * @param {Zotero.Item} item
-     * @return {Promise<Zotero.Item>} - New item
-     */
+   * Retrieves metadata for a PDF and saves it as an item
+   * @param {Zotero.Item} item
+   * @return {Promise<Zotero.Item>} - New item
+   */
   async function _recognize(item) {
     if (Zotero.RecognizePDF.recognizeStub) {
       return Zotero.RecognizePDF.recognizeStub(item);
@@ -419,6 +417,7 @@ Zotero.RecognizePDF = new function () {
     if (!filePath || !(await OS.File.exists(filePath))) throw new Zotero.Exception.Alert('recognizePDF.fileNotFound');
 
     let json = await extractJSON(filePath, MAX_PAGES);
+    json.fileName = OS.Path.basename(filePath);
 
     let containingTextPages = 0;
 
@@ -507,7 +506,7 @@ Zotero.RecognizePDF = new function () {
           // translate.js, but for ISBNs we just get the data (libraryID=false) and do the
           // saving manually.
           if (Zotero.Prefs.get('automaticTags')) {
-            translatedItems[0].tags = translatedItems[0].tags.map(tag => {
+            translatedItems[0].tags = translatedItems[0].tags.map((tag) => {
               if (typeof tag == 'string') {
                 return {
                   tag,
@@ -586,9 +585,9 @@ Zotero.RecognizePDF = new function () {
   }
 
   /**
-     * To customize the recognizer endpoint, set either recognize.url (used directly)
-     * or services.url (used with a 'recognizer/' suffix).
-     */
+   * To customize the recognizer endpoint, set either recognize.url (used directly)
+   * or services.url (used with a 'recognizer/' suffix).
+   */
   function _getBaseURL() {
     var url = Zotero.Prefs.get("recognize.url");
     if (url) {
