@@ -9,6 +9,10 @@ Zotero.Sync.Storage.Local = {
   SYNC_STATE_FORCE_DOWNLOAD: 4,
   SYNC_STATE_IN_CONFLICT: 5,
 
+  // If last-known remaining storage is below this number of megabytes, skip further upload
+  // attempts in various situations
+  STORAGE_REMAINING_MINIMUM: 5,
+
   lastFullFileCheck: {},
   uploadCheckFiles: [],
   storageRemainingForLibrary: new Map(),
@@ -99,13 +103,13 @@ Zotero.Sync.Storage.Local = {
   },
 
   /**
-      * Check or enable download-as-needed mode
-      *
-      * @param {Integer} [libraryID]
-      * @param {Boolean} [enable] - If true, enable download-as-needed mode for the given library
-      * @return {Boolean|undefined} - If 'enable' isn't set to true, return true if
-      *     download-as-needed mode enabled and false if not
-      */
+   * Check or enable download-as-needed mode
+   *
+   * @param {Integer} [libraryID]
+   * @param {Boolean} [enable] - If true, enable download-as-needed mode for the given library
+   * @return {Boolean|undefined} - If 'enable' isn't set to true, return true if
+   *     download-as-needed mode enabled and false if not
+   */
   downloadAsNeeded: function (libraryID, enable) {
     var pref = this._getDownloadPrefFromLibrary(libraryID);
     var val = 'on-demand';
@@ -117,13 +121,13 @@ Zotero.Sync.Storage.Local = {
   },
 
   /**
-      * Check or enable download-on-sync mode
-      *
-      * @param {Integer} [libraryID]
-      * @param {Boolean} [enable] - If true, enable download-on-demand mode for the given library
-      * @return {Boolean|undefined} - If 'enable' isn't set to true, return true if
-      *     download-as-needed mode enabled and false if not
-      */
+   * Check or enable download-on-sync mode
+   *
+   * @param {Integer} [libraryID]
+   * @param {Boolean} [enable] - If true, enable download-on-demand mode for the given library
+   * @return {Boolean|undefined} - If 'enable' isn't set to true, return true if
+   *     download-as-needed mode enabled and false if not
+   */
   downloadOnSync: function (libraryID, enable) {
     var pref = this._getDownloadPrefFromLibrary(libraryID);
     var val = 'on-sync';
@@ -145,10 +149,10 @@ Zotero.Sync.Storage.Local = {
   },
 
   /**
-      * Get files to check for local modifications for uploading
-      *
-      * This includes files previously modified or opened externally via Zotero within maxCheckAge
-      */
+   * Get files to check for local modifications for uploading
+   *
+   * This includes files previously modified or opened externally via Zotero within maxCheckAge
+   */
   getFilesToCheck: Zotero.Promise.coroutine(function* (libraryID, maxCheckAge) {
     var minTime = new Date().getTime() - maxCheckAge * 1000;
 
@@ -167,7 +171,7 @@ Zotero.Sync.Storage.Local = {
 
     // Get files opened since maxCheckAge
     itemIDs = itemIDs.concat(
-    this.uploadCheckFiles.filter(x => x.timestamp >= minTime).map(x => x.itemID));
+    this.uploadCheckFiles.filter((x) => x.timestamp >= minTime).map((x) => x.itemID));
 
 
     return Zotero.Utilities.arrayUnique(itemIDs);
@@ -175,20 +179,20 @@ Zotero.Sync.Storage.Local = {
 
 
   /**
-       * Scans local files and marks any that have changed for uploading
-       * and any that are missing for downloading
-       *
-       * @param {Integer} libraryID
-       * @param {Integer[]} [itemIDs]
-       * @param {Object} [itemModTimes]  Item mod times indexed by item ids;
-       *                                 items with stored mod times
-       *                                 that differ from the provided
-       *                                 time but file mod times
-       *                                 matching the stored time will
-       *                                 be marked for download
-       * @return {Promise} Promise resolving to TRUE if any items changed state,
-       *                   FALSE otherwise
-       */
+   * Scans local files and marks any that have changed for uploading
+   * and any that are missing for downloading
+   *
+   * @param {Integer} libraryID
+   * @param {Integer[]} [itemIDs]
+   * @param {Object} [itemModTimes]  Item mod times indexed by item ids;
+   *                                 items with stored mod times
+   *                                 that differ from the provided
+   *                                 time but file mod times
+   *                                 matching the stored time will
+   *                                 be marked for download
+   * @return {Promise} Promise resolving to TRUE if any items changed state,
+   *                   FALSE otherwise
+   */
   checkForUpdatedFiles: Zotero.Promise.coroutine(function* (libraryID, itemIDs, itemModTimes) {
     var libraryName = Zotero.Libraries.getName(libraryID);
     var msg = "Checking for locally changed attachment files in " + libraryName;
@@ -328,6 +332,17 @@ Zotero.Sync.Storage.Local = {
 
     try {
       file = yield OS.File.open(path);
+
+      // If file is already marked for upload, skip check. Even if the file was changed
+      // both locally and remotely, conflicts are checked at upload time, so we don't need
+      // to worry about it here.
+      //
+      // This is after open() so that a missing file is properly marked for download.
+      if (item.attachmentSyncState == this.SYNC_STATE_TO_UPLOAD) {
+        Zotero.debug("File is already marked for upload");
+        return false;
+      }
+
       let info = yield file.stat();
       //Zotero.debug("Memory usage: " + memmgr.resident);
 
@@ -339,14 +354,6 @@ Zotero.Sync.Storage.Local = {
         fmtime = 0;
       }
 
-      // If file is already marked for upload, skip check. Even if the file was changed
-      // both locally and remotely, conflicts are checked at upload time, so we don't need
-      // to worry about it here.
-      if (item.attachmentSyncState == this.SYNC_STATE_TO_UPLOAD) {
-        Zotero.debug("File is already marked for upload");
-        return false;
-      }
-
       //Zotero.debug("Stored mtime is " + attachmentData.mtime);
       //Zotero.debug("File mtime is " + fmtime);
 
@@ -429,13 +436,13 @@ Zotero.Sync.Storage.Local = {
   }),
 
   /**
-       *
-       * @param {Zotero.Item} item
-       * @param {Integer} fmtime - File modification time in milliseconds
-       * @param {Integer} mtime - Remote modification time in milliseconds
-       * @return {Boolean} - True if file modification time differs from remote mod time,
-       *                     false otherwise
-       */
+   *
+   * @param {Zotero.Item} item
+   * @param {Integer} fmtime - File modification time in milliseconds
+   * @param {Integer} mtime - Remote modification time in milliseconds
+   * @return {Boolean} - True if file modification time differs from remote mod time,
+   *                     false otherwise
+   */
   checkFileModTime: function (item, fmtime, mtime) {
     var libraryKey = item.libraryKey;
 
@@ -445,18 +452,18 @@ Zotero.Sync.Storage.Local = {
     // Compare floored timestamps for filesystems that don't support millisecond
     // precision (e.g., HFS+)
     else if (Math.floor(mtime / 1000) == Math.floor(fmtime / 1000)) {
-        Zotero.debug(`File mod times for ${libraryKey} are within one-second precision ` +
-        "(" + fmtime + " \u2248 " + mtime + ") -- skipping");
-      }
-      // Allow timestamp to be exactly one hour off to get around time zone issues
-      // -- there may be a proper way to fix this
-      else if (Math.abs(Math.floor(fmtime / 1000) - Math.floor(mtime / 1000)) == 3600) {
-          Zotero.debug(`File mod time (${fmtime}) for {$libraryKey} is exactly one hour off ` +
-          `remote file (${mtime}) -- assuming time zone issue and skipping`);
-        } else
-        {
-          return true;
-        }
+      Zotero.debug(`File mod times for ${libraryKey} are within one-second precision ` +
+      "(" + fmtime + " \u2248 " + mtime + ") -- skipping");
+    }
+    // Allow timestamp to be exactly one hour off to get around time zone issues
+    // -- there may be a proper way to fix this
+    else if (Math.abs(Math.floor(fmtime / 1000) - Math.floor(mtime / 1000)) == 3600) {
+      Zotero.debug(`File mod time (${fmtime}) for {$libraryKey} is exactly one hour off ` +
+      `remote file (${mtime}) -- assuming time zone issue and skipping`);
+    } else
+    {
+      return true;
+    }
 
     return false;
   },
@@ -472,11 +479,11 @@ Zotero.Sync.Storage.Local = {
 
 
   /**
-       * Get files marked as ready to download
-       *
-       * @param {Integer} libraryID
-       * @return {Promise<Number[]>} - Promise for an array of attachment itemIDs
-       */
+   * Get files marked as ready to download
+   *
+   * @param {Integer} libraryID
+   * @return {Promise<Number[]>} - Promise for an array of attachment itemIDs
+   */
   getFilesToDownload: function (libraryID, forcedOnly) {
     var sql = "SELECT itemID FROM itemAttachments JOIN items USING (itemID) " +
     "WHERE libraryID=? AND syncState IN (?";
@@ -495,29 +502,30 @@ Zotero.Sync.Storage.Local = {
 
 
   /**
-      * Get files marked as ready to upload
-      *
-      * @param {Integer} libraryID
-      * @return {Promise<Number[]>} - Promise for an array of attachment itemIDs
-      */
+   * Get files marked as ready to upload
+   *
+   * @param {Integer} libraryID
+   * @return {Promise<Number[]>} - Promise for an array of attachment itemIDs
+   */
   getFilesToUpload: function (libraryID) {
     var sql = "SELECT itemID FROM itemAttachments JOIN items USING (itemID) " +
-    "WHERE libraryID=? AND syncState IN (?,?) AND linkMode IN (?,?)";
+    "WHERE libraryID=? AND syncState IN (?,?) AND linkMode IN (?,?,?)";
     var params = [
     libraryID,
     this.SYNC_STATE_TO_UPLOAD,
     this.SYNC_STATE_FORCE_UPLOAD,
     Zotero.Attachments.LINK_MODE_IMPORTED_FILE,
-    Zotero.Attachments.LINK_MODE_IMPORTED_URL];
+    Zotero.Attachments.LINK_MODE_IMPORTED_URL,
+    Zotero.Attachments.LINK_MODE_EMBEDDED_IMAGE];
 
     return Zotero.DB.columnQueryAsync(sql, params);
   },
 
 
   /**
-      * @param {Integer} libraryID
-      * @return {Promise<String[]>} - Promise for an array of item keys
-      */
+   * @param {Integer} libraryID
+   * @return {Promise<String[]>} - Promise for an array of item keys
+   */
   getDeletedFiles: function (libraryID) {
     var sql = "SELECT key FROM storageDeleteLog WHERE libraryID=?";
     return Zotero.DB.columnQueryAsync(sql, libraryID);
@@ -525,10 +533,10 @@ Zotero.Sync.Storage.Local = {
 
 
   /**
-      * @param {Zotero.Item[]} items
-      * @param {String|Integer} syncState
-      * @return {Promise}
-      */
+   * @param {Zotero.Item[]} items
+   * @param {String|Integer} syncState
+   * @return {Promise}
+   */
   updateSyncStates: function (items, syncState) {
     if (syncState === undefined) {
       throw new Error("Sync state not specified");
@@ -540,12 +548,12 @@ Zotero.Sync.Storage.Local = {
     items,
     1000,
     async function (chunk) {
-      chunk.forEach(item => {
+      chunk.forEach((item) => {
         item._attachmentSyncState = syncState;
       });
       return Zotero.DB.queryAsync(
       "UPDATE itemAttachments SET syncState=? WHERE itemID IN " +
-      "(" + chunk.map(item => item.id).join(', ') + ")",
+      "(" + chunk.map((item) => item.id).join(', ') + ")",
       syncState);
 
     });
@@ -554,11 +562,11 @@ Zotero.Sync.Storage.Local = {
 
 
   /**
-      * Mark all stored files for upload checking
-      *
-      * This is used when switching between storage modes in the preferences so that all existing files
-      * are uploaded via the new mode if necessary.
-      */
+   * Mark all stored files for upload checking
+   *
+   * This is used when switching between storage modes in the preferences so that all existing files
+   * are uploaded via the new mode if necessary.
+   */
   resetAllSyncStates: async function (libraryID) {
     if (!libraryID) {
       throw new Error("libraryID not provided");
@@ -566,12 +574,13 @@ Zotero.Sync.Storage.Local = {
 
     return Zotero.DB.executeTransaction(async function () {
       var sql = "SELECT itemID FROM items JOIN itemAttachments USING (itemID) " +
-      "WHERE libraryID=? AND itemTypeID=? AND linkMode IN (?, ?)";
+      "WHERE libraryID=? AND itemTypeID=? AND linkMode IN (?, ?, ?)";
       var params = [
       libraryID,
       Zotero.ItemTypes.getID('attachment'),
       Zotero.Attachments.LINK_MODE_IMPORTED_FILE,
-      Zotero.Attachments.LINK_MODE_IMPORTED_URL];
+      Zotero.Attachments.LINK_MODE_IMPORTED_URL,
+      Zotero.Attachments.LINK_MODE_EMBEDDED_IMAGE];
 
       var itemIDs = await Zotero.DB.columnQueryAsync(sql, params);
       for (let itemID of itemIDs) {
@@ -585,14 +594,14 @@ Zotero.Sync.Storage.Local = {
 
 
   /**
-      * Extract a downloaded file and update the database metadata
-      *
-      * @param {Zotero.Item} data.item
-      * @param {Integer}     data.mtime
-      * @param {String}      data.md5
-      * @param {Boolean}     data.compressed
-      * @return {Promise}
-      */
+   * Extract a downloaded file and update the database metadata
+   *
+   * @param {Zotero.Item} data.item
+   * @param {Integer}     data.mtime
+   * @param {String}      data.md5
+   * @param {Boolean}     data.compressed
+   * @return {Promise}
+   */
   processDownload: Zotero.Promise.coroutine(function* (data) {
     if (!data) {
       throw new Error("'data' not set");
@@ -845,7 +854,7 @@ Zotero.Sync.Storage.Local = {
 
       // Make sure all components of the path are valid, in case an invalid character somehow made
       // it into the ZIP (e.g., from before we checked for them)
-      var filteredPath = filePath.split('/').map(part => Zotero.File.getValidFileName(part)).join('/');
+      var filteredPath = filePath.split('/').map((part) => Zotero.File.getValidFileName(part)).join('/');
       if (filteredPath != filePath) {
         Zotero.debug("Filtering filename '" + filePath + "' to '" + filteredPath + "'");
         filePath = filteredPath;
@@ -960,8 +969,8 @@ Zotero.Sync.Storage.Local = {
 
 
   /**
-       * @return {Promise<Object[]>} - A promise for an array of conflict objects
-       */
+   * @return {Promise<Object[]>} - A promise for an array of conflict objects
+   */
   getConflicts: Zotero.Promise.coroutine(function* (libraryID) {
     var sql = "SELECT itemID, version FROM items JOIN itemAttachments USING (itemID) " +
     "WHERE libraryID=? AND syncState=?";
@@ -982,10 +991,10 @@ Zotero.Sync.Storage.Local = {
     if (!cacheObjects.length) return [];
 
     var cacheObjectsByKey = {};
-    cacheObjects.forEach(obj => cacheObjectsByKey[obj.key] = obj);
+    cacheObjects.forEach((obj) => cacheObjectsByKey[obj.key] = obj);
 
     var items = [];
-    var localItems = yield Zotero.Items.getAsync(rows.map(row => row.itemID));
+    var localItems = yield Zotero.Items.getAsync(rows.map((row) => row.itemID));
     for (let localItem of localItems) {
       // Use the mtime for the dateModified field, since that's all that's shown in the
       // CR window at the moment
@@ -1068,8 +1077,8 @@ Zotero.Sync.Storage.Local = {
         }
         // Remote
         else {
-            syncState = this.SYNC_STATE_FORCE_DOWNLOAD;
-          }
+          syncState = this.SYNC_STATE_FORCE_DOWNLOAD;
+        }
         item.attachmentSyncState = syncState;
         yield item.save({ skipAll: true });
       }
