@@ -31,12 +31,12 @@ if (!Zotero.Sync.Data) {
 Zotero.Sync.Data.conflictDelayIntervals = [10000, 20000, 40000, 60000, 120000, 240000, 300000];
 
 /**
-                                                                                                 * An Engine manages sync processes for a given library
-                                                                                                 *
-                                                                                                 * @param {Object} options
-                                                                                                 * @param {Zotero.Sync.APIClient} options.apiClient
-                                                                                                 * @param {Integer} options.libraryID
-                                                                                                 */
+ * An Engine manages sync processes for a given library
+ *
+ * @param {Object} options
+ * @param {Zotero.Sync.APIClient} options.apiClient
+ * @param {Integer} options.libraryID
+ */
 Zotero.Sync.Data.Engine = function (options) {
   if (options.apiClient == undefined) {
     throw new Error("options.apiClient not set");
@@ -67,7 +67,7 @@ Zotero.Sync.Data.Engine = function (options) {
   'resetMode'];
 
   this.options = {};
-  this.optionNames.forEach(x => {
+  this.optionNames.forEach((x) => {
     // Create dummy functions if not set
     if (x == 'setStatus' || x == 'onError') {
       this[x] = options[x] || function () {};
@@ -149,6 +149,7 @@ Zotero.Sync.Data.Engine.prototype.start = Zotero.Promise.coroutine(function* ()
         throw e;
       }
       Zotero.debug("Upload failed -- performing download", 2);
+      Zotero.debug(e, 1);
       downloadResult = yield this._startDownload();
       Zotero.debug("Download result is " + downloadResult, 4);
       throw e;
@@ -194,11 +195,11 @@ Zotero.Sync.Data.Engine.prototype.start = Zotero.Promise.coroutine(function* ()
         // After the first upload version conflict (which is expected after remote changes),
         // start delaying to give other sync sessions time to complete
         else {
-            let keepGoing = yield gen.next().value;
-            if (!keepGoing) {
-              throw new Error("Could not sync " + this.library.name + " -- too many retries");
-            }
+          let keepGoing = yield gen.next().value;
+          if (!keepGoing) {
+            throw new Error("Could not sync " + this.library.name + " -- too many retries");
           }
+        }
 
         downloadResult = yield this._startDownload();
         Zotero.debug("Download result is " + downloadResult, 4);
@@ -223,13 +224,22 @@ Zotero.Sync.Data.Engine.prototype.start = Zotero.Promise.coroutine(function* ()
     skipNotifier: true });
 
 
+  if (this.library.libraryType == 'group') {
+    try {
+      yield this._updateGroupItemUsers();
+    }
+    catch (e) {
+      Zotero.logError(e);
+    }
+  }
+
   Zotero.debug("Done syncing " + this.library.name);
 });
 
 
 /**
-     * Stop the sync process
-     */
+ * Stop the sync process
+ */
 Zotero.Sync.Data.Engine.prototype.stop = function () {
   Zotero.debug("Stopping data sync for " + this.library.name);
   this._stopping = true;
@@ -237,10 +247,10 @@ Zotero.Sync.Data.Engine.prototype.stop = function () {
 
 
 /**
-    * Download updated objects from API and save to DB
-    *
-    * @return {Promise<Integer>} - A download result code (this.DOWNLOAD_RESULT_*)
-    */
+ * Download updated objects from API and save to DB
+ *
+ * @return {Promise<Integer>} - A download result code (this.DOWNLOAD_RESULT_*)
+ */
 Zotero.Sync.Data.Engine.prototype._startDownload = Zotero.Promise.coroutine(function* () {
   var localChanges = false;
   var libraryVersion = this.library.libraryVersion;
@@ -346,18 +356,18 @@ Zotero.Sync.Data.Engine.prototype._startDownload = Zotero.Promise.coroutine(func
 
 
 /**
-     * Download settings modified since the given version
-     *
-     * Unlike the other download methods, this method, which runs first in the main download process,
-     * returns an object rather than just a download result code. It does this so it can return the
-     * current library version from the API to pass to later methods, allowing them to restart the download
-     * process if there was a remote change.
-     *
-     * @param {Integer} since - Last-known library version; get changes since this version
-     * @param {Integer} [newLibraryVersion] - Newest library version seen in this sync process; if newer
-     *     version is seen, restart the sync
-     * @return {Object} - Object with 'result' (DOWNLOAD_RESULT_*) and 'libraryVersion'
-     */
+ * Download settings modified since the given version
+ *
+ * Unlike the other download methods, this method, which runs first in the main download process,
+ * returns an object rather than just a download result code. It does this so it can return the
+ * current library version from the API to pass to later methods, allowing them to restart the download
+ * process if there was a remote change.
+ *
+ * @param {Integer} since - Last-known library version; get changes since this version
+ * @param {Integer} [newLibraryVersion] - Newest library version seen in this sync process; if newer
+ *     version is seen, restart the sync
+ * @return {Object} - Object with 'result' (DOWNLOAD_RESULT_*) and 'libraryVersion'
+ */
 Zotero.Sync.Data.Engine.prototype._downloadSettings = Zotero.Promise.coroutine(function* (since, newLibraryVersion) {
   let results = yield this.apiClient.getSettings(
   this.library.libraryType,
@@ -402,15 +412,15 @@ Zotero.Sync.Data.Engine.prototype._downloadSettings = Zotero.Promise.coroutine(f
 
 
 /**
-     * Get versions of objects updated remotely since the last sync time and kick off object downloading
-     *
-     * @param {String} objectType
-     * @param {Integer} since - Last-known library version; get changes sinces this version
-     * @param {Integer} newLibraryVersion - Last library version seen in this sync process; if newer version
-     *     is seen, restart the sync
-     * @param {Object} [options]
-     * @return {Promise<Integer>} - A download result code (this.DOWNLOAD_RESULT_*)
-     */
+ * Get versions of objects updated remotely since the last sync time and kick off object downloading
+ *
+ * @param {String} objectType
+ * @param {Integer} since - Last-known library version; get changes sinces this version
+ * @param {Integer} newLibraryVersion - Last library version seen in this sync process; if newer version
+ *     is seen, restart the sync
+ * @param {Object} [options]
+ * @return {Promise<Integer>} - A download result code (this.DOWNLOAD_RESULT_*)
+ */
 Zotero.Sync.Data.Engine.prototype._downloadUpdatedObjects = Zotero.Promise.coroutine(function* (objectType, since, newLibraryVersion, options = {}) {
   var objectTypePlural = Zotero.DataObjectUtilities.getObjectTypePlural(objectType);
   var objectsClass = Zotero.DataObjectUtilities.getObjectsClassForObjectType(objectType);
@@ -519,38 +529,43 @@ Zotero.Sync.Data.Engine.prototype._downloadUpdatedObjects = Zotero.Promise.corou
 
 
 /**
-     * Download data for specified objects from the API and run processing on them, and show the conflict
-     * resolution window if necessary
-     *
-     * @return {Promise<Integer>} - A download result code (this.DOWNLOAD_RESULT_*)
-     */
+ * Download data for specified objects from the API and run processing on them, and show the conflict
+ * resolution window if necessary
+ *
+ * @return {Promise<Integer>} - A download result code (this.DOWNLOAD_RESULT_*)
+ */
 Zotero.Sync.Data.Engine.prototype._downloadObjects = async function (objectType, keys) {
   var objectTypePlural = Zotero.DataObjectUtilities.getObjectTypePlural(objectType);
 
   var remainingKeys = [...keys];
   var lastLength = keys.length;
   var objectData = {};
-  keys.forEach(key => objectData[key] = null);
+  keys.forEach((key) => objectData[key] = null);
 
   while (true) {
     this._statusCheck();
 
     // Get data we've downloaded in a previous loop but failed to process
-    var json = [];
+    var results = [];
     let keysToDownload = [];
+    var keysToReprocess = [];
     for (let key in objectData) {
       if (objectData[key] === null) {
         keysToDownload.push(key);
       } else
       {
-        json.push(objectData[key]);
+        keysToReprocess.push(key);
+
       }
     }
-    if (json.length) {
-      json = [json];
+    if (keysToReprocess) {
+      results.push({
+        keys: keysToReprocess,
+        json: keysToReprocess.map((key) => objectData[key]) });
+
     }
     // Add promises for batches of downloaded data for remaining keys
-    json.push(...this.apiClient.downloadObjects(
+    results.push(...this.apiClient.downloadObjects(
     this.library.libraryType,
     this.libraryTypeID,
     objectType,
@@ -566,33 +581,43 @@ Zotero.Sync.Data.Engine.prototype._downloadObjects = async function (objectType,
     " in " + this.library.name);
 
 
+    var missingKeys = [];
     var conflicts = [];
     var restored = [];
     var num = 0;
 
     // Process batches of object data as they're available, one at a time
     await Zotero.Promise.map(
-    json,
-    async function (batch) {
+    results,
+    async function ({ keys: batchKeys, json, error }) {
       this._statusCheck();
 
       Zotero.debug(`Processing batch of downloaded ${objectTypePlural} in ${this.library.name}`);
 
-      if (!Array.isArray(batch)) {
-        this.failed = batch;
+      if (error) {
+        this.failed = error;
         return;
       }
 
       // Save downloaded JSON for later attempts
-      batch.forEach(obj => {
+      var seenKeys = new Set();
+      json.forEach((obj) => {
         objectData[obj.key] = obj;
+        seenKeys.add(obj.key);
       });
+      // If a key we requested wasn't returned, it's missing remotely
+      for (let key of batchKeys) {
+        if (!seenKeys.has(key)) {
+          missingKeys.push(key);
+          objectData[key] = false;
+        }
+      }
 
       // Process objects
       let results = await Zotero.Sync.Data.Local.processObjectsFromJSON(
       objectType,
       this.libraryID,
-      batch,
+      json,
       this._getOptions({
         onObjectProcessed: () => {
           num++;
@@ -617,7 +642,7 @@ Zotero.Sync.Data.Engine.prototype._downloadObjects = async function (objectType,
           {
             size = 50;
           }
-          return Math.min(size, batch.length);
+          return Math.min(size, json.length);
         } }));
 
 
@@ -625,7 +650,7 @@ Zotero.Sync.Data.Engine.prototype._downloadObjects = async function (objectType,
       num += results.length;
       let processedKeys = [];
       let conflictResults = [];
-      results.forEach(x => {
+      results.forEach((x) => {
         // If data was processed, remove JSON
         if (x.processed) {
           delete objectData[x.key];
@@ -661,20 +686,21 @@ Zotero.Sync.Data.Engine.prototype._downloadObjects = async function (objectType,
 
     // If all requests were successful, such that we had a chance to see all keys, remove keys we
     // didn't see from the sync queue so they don't keep being retried forever
-    if (!this.failed) {
-      let missingKeys = keys.filter(key => objectData[key] === null);
-      if (missingKeys.length) {
-        Zotero.debug(`Removing ${missingKeys.length} missing ` +
-        Zotero.Utilities.pluralize(missingKeys.length, [objectType, objectTypePlural]) +
-        " from sync queue");
-        await Zotero.Sync.Data.Local.removeObjectsFromSyncQueue(objectType, this.libraryID, missingKeys);
-        remainingKeys = Zotero.Utilities.arrayDiff(remainingKeys, missingKeys);
-      }
+    if (missingKeys.length) {
+      Zotero.debug(`Removing ${missingKeys.length} missing ` +
+      Zotero.Utilities.pluralize(missingKeys.length, [objectType, objectTypePlural]) +
+      " from sync queue");
+      await Zotero.Sync.Data.Local.removeObjectsFromSyncQueue(objectType, this.libraryID, missingKeys);
+      remainingKeys = Zotero.Utilities.arrayDiff(remainingKeys, missingKeys);
     }
 
     if (!remainingKeys.length || remainingKeys.length == lastLength) {
       // Add failed objects to sync queue
-      let failedKeys = keys.filter(key => objectData[key]);
+      //
+      // Object failed if it's still present in the object data (since successfully processed
+      // objects are removed) and it's not false (meaning it was missing from the request,
+      // which isn't a failure)
+      let failedKeys = keys.filter((key) => objectData[key] !== undefined && objectData[key] !== false);
       if (failedKeys.length) {
         Zotero.debug(`Queueing ${failedKeys.length} failed ` +
         Zotero.Utilities.pluralize(failedKeys.length, [objectType, objectTypePlural]) +
@@ -711,9 +737,9 @@ Zotero.Sync.Data.Engine.prototype._downloadObjects = async function (objectType,
     let results = await Zotero.Sync.Data.Local.processConflicts(
     objectType, this.libraryID, conflicts, this._getOptions());
 
-    let keys = results.filter(x => x.processed).map(x => x.key);
+    let keys = results.filter((x) => x.processed).map((x) => x.key);
     // If all keys are unprocessed and didn't fail from an error, conflict resolution was cancelled
-    if (results.every(x => !x.processed && !x.error)) {
+    if (results.every((x) => !x.processed && !x.error)) {
       throw new Zotero.Sync.UserCancelledException();
     }
     await Zotero.Sync.Data.Local.removeObjectsFromSyncQueue(objectType, this.libraryID, keys);
@@ -724,15 +750,15 @@ Zotero.Sync.Data.Engine.prototype._downloadObjects = async function (objectType,
 
 
 /**
-    * If a collection is deleted locally but modified remotely between syncs, the local collection is
-    * restored, but collection membership is a property of items, the local items that were previously
-    * in that collection won't be any longer (or they might have been deleted along with the collection),
-    * so we have to get the current collection items from the API and either add them back
-    * (if they exist) or clear them from the delete log and mark them for download.
-    *
-    * Remote items in the trash aren't currently restored and will be removed from the collection when the
-    * local collection-item removal syncs up.
-    */
+ * If a collection is deleted locally but modified remotely between syncs, the local collection is
+ * restored, but collection membership is a property of items, the local items that were previously
+ * in that collection won't be any longer (or they might have been deleted along with the collection),
+ * so we have to get the current collection items from the API and either add them back
+ * (if they exist) or clear them from the delete log and mark them for download.
+ *
+ * Remote items in the trash aren't currently restored and will be removed from the collection when the
+ * local collection-item removal syncs up.
+ */
 Zotero.Sync.Data.Engine.prototype._restoreRestoredCollectionItems = async function (collectionKeys) {
   for (let collectionKey of collectionKeys) {
     let { keys: itemKeys } = await this.apiClient.getKeys(
@@ -790,13 +816,13 @@ Zotero.Sync.Data.Engine.prototype._restoreRestoredCollectionItems = async functi
 
 
 /**
-    * Get deleted objects from the API and process them
-    *
-    * @param {Integer} since - Last-known library version; get changes sinces this version
-    * @param {Integer} [newLibraryVersion] - Newest library version seen in this sync process; if newer
-    *     version is seen, restart the sync
-    * @return {Object} - Object with 'result' (DOWNLOAD_RESULT_*) and 'libraryVersion'
-    */
+ * Get deleted objects from the API and process them
+ *
+ * @param {Integer} since - Last-known library version; get changes sinces this version
+ * @param {Integer} [newLibraryVersion] - Newest library version seen in this sync process; if newer
+ *     version is seen, restart the sync
+ * @return {Object} - Object with 'result' (DOWNLOAD_RESULT_*) and 'libraryVersion'
+ */
 Zotero.Sync.Data.Engine.prototype._downloadDeletions = Zotero.Promise.coroutine(function* (since, newLibraryVersion) {
   const batchSize = 50;
 
@@ -859,22 +885,22 @@ Zotero.Sync.Data.Engine.prototype._downloadDeletions = Zotero.Promise.coroutine(
       }
       // Conflict resolution
       else if (objectType == 'item') {
-          // If item is already in trash locally, just delete it
-          if (obj.deleted) {
-            Zotero.debug("Local item is in trash -- applying remote deletion");
-            yield obj.eraseTx({
-              skipDeleteLog: true });
-
-            continue;
-          }
-          conflicts.push({
-            libraryID: this.libraryID,
-            left: obj.toJSON(),
-            right: {
-              deleted: true } });
-
+        // If item is already in trash locally, just delete it
+        if (obj.deleted) {
+          Zotero.debug("Local item is in trash -- applying remote deletion");
+          yield obj.eraseTx({
+            skipDeleteLog: true });
 
+          continue;
         }
+        conflicts.push({
+          libraryID: this.libraryID,
+          left: obj.toJSON(),
+          right: {
+            deleted: true } });
+
+
+      }
     }
 
     if (conflicts.length) {
@@ -946,9 +972,9 @@ Zotero.Sync.Data.Engine.prototype._downloadDeletions = Zotero.Promise.coroutine(
 
 
 /**
-     * If something else modified the remote library while we were getting updates, wait for increasing
-     * amounts of time before trying again, and then start from the beginning
-     */
+ * If something else modified the remote library while we were getting updates, wait for increasing
+ * amounts of time before trying again, and then start from the beginning
+ */
 Zotero.Sync.Data.Engine.prototype._onLibraryVersionChange = Zotero.Promise.coroutine(function* (mode) {
   Zotero.logError("Library version changed since last download -- restarting sync");
 
@@ -966,11 +992,11 @@ Zotero.Sync.Data.Engine.prototype._onLibraryVersionChange = Zotero.Promise.corou
 
 
 /**
-     * Get unsynced objects, build upload JSON, and start API requests
-     *
-     * @throws {Zotero.HTTP.UnexpectedStatusException}
-     * @return {Promise<Integer>} - An upload result code (this.UPLOAD_RESULT_*)
-     */
+ * Get unsynced objects, build upload JSON, and start API requests
+ *
+ * @throws {Zotero.HTTP.UnexpectedStatusException}
+ * @return {Promise<Integer>} - An upload result code (this.UPLOAD_RESULT_*)
+ */
 Zotero.Sync.Data.Engine.prototype._startUpload = Zotero.Promise.coroutine(function* () {
   var libraryVersion = this.library.libraryVersion;
 
@@ -1006,7 +1032,7 @@ Zotero.Sync.Data.Engine.prototype._startUpload = Zotero.Promise.coroutine(functi
 
     // Skip objects in sync queue, because they might have unresolved conflicts.
     // The queue only has keys, so we have to convert to keys and back.
-    let unsyncedKeys = ids.map(id => objectsClass.getLibraryAndKeyFromID(id).key);
+    let unsyncedKeys = ids.map((id) => objectsClass.getLibraryAndKeyFromID(id).key);
     let origUnsynced = unsyncedKeys; // TEMP
     let queueKeys = yield Zotero.Sync.Data.Local.getObjectsFromSyncQueue(objectType, this.libraryID);
     let newUnsyncedKeys = Zotero.Utilities.arrayDiff(unsyncedKeys, queueKeys);
@@ -1019,7 +1045,7 @@ Zotero.Sync.Data.Engine.prototype._startUpload = Zotero.Promise.coroutine(functi
     // TEMP
     //ids = unsyncedKeys.map(key => objectsClass.getIDFromLibraryAndKey(this.libraryID, key));
     let missing = [];
-    ids = unsyncedKeys.map(key => {
+    ids = unsyncedKeys.map((key) => {
       let id = objectsClass.getIDFromLibraryAndKey(this.libraryID, key);
       if (!id) {
         Zotero.debug("Missing id for key " + key);
@@ -1029,8 +1055,8 @@ Zotero.Sync.Data.Engine.prototype._startUpload = Zotero.Promise.coroutine(functi
     });
     if (missing.length) {
       Zotero.debug("Missing " + objectTypePlural + ":");
-      Zotero.debug(ids.filter(id => !id));
-      Zotero.debug(ids.filter(id => !objectsClass.getLibraryAndKeyFromID(id)));
+      Zotero.debug(ids.filter((id) => !id));
+      Zotero.debug(ids.filter((id) => !objectsClass.getLibraryAndKeyFromID(id)));
       Zotero.debug(missing);
     }
 
@@ -1092,35 +1118,42 @@ Zotero.Sync.Data.Engine.prototype._startUpload = Zotero.Promise.coroutine(functi
 });
 
 
-Zotero.Sync.Data.Engine.prototype._uploadSettings = Zotero.Promise.coroutine(function* (settings, libraryVersion) {
-  let json = {};
-  for (let key in settings) {
-    json[key] = {
-      value: settings[key] };
-
-  }
-  libraryVersion = yield this.apiClient.uploadSettings(
-  this.library.libraryType,
-  this.libraryTypeID,
-  libraryVersion,
-  json);
-
-  yield Zotero.SyncedSettings.markAsSynced(
-  this.libraryID,
+Zotero.Sync.Data.Engine.prototype._uploadSettings = async function (settings, libraryVersion) {
+  const SETTINGS_BATCH_SIZE = 250;
+  await Zotero.Utilities.Internal.forEachChunkAsync(
   Object.keys(settings),
-  libraryVersion);
+  SETTINGS_BATCH_SIZE,
+  async function (keys) {
+    let json = {};
+    for (let key of keys) {
+      json[key] = {
+        value: settings[key] };
 
-  if (this.library.libraryVersion == this.library.storageVersion) {
-    this.library.storageVersion = libraryVersion;
-  }
-  this.library.libraryVersion = libraryVersion;
-  yield this.library.saveTx({
-    skipNotifier: true });
+    }
+    libraryVersion = await this.apiClient.uploadSettings(
+    this.library.libraryType,
+    this.libraryTypeID,
+    libraryVersion,
+    json);
 
+    await Zotero.SyncedSettings.markAsSynced(
+    this.libraryID,
+    keys,
+    libraryVersion);
+
+
+    if (this.library.libraryVersion == this.library.storageVersion) {
+      this.library.storageVersion = libraryVersion;
+    }
+    this.library.libraryVersion = libraryVersion;
+    await this.library.saveTx({
+      skipNotifier: true });
+
+  }.bind(this));
 
   Zotero.debug("Done uploading settings in " + this.library.name);
   return libraryVersion;
-});
+};
 
 
 Zotero.Sync.Data.Engine.prototype._uploadObjects = Zotero.Promise.coroutine(function* (objectType, ids, libraryVersion) {
@@ -1145,19 +1178,19 @@ Zotero.Sync.Data.Engine.prototype._uploadObjects = Zotero.Promise.coroutine(func
     notify: function (event, type, ids, extraData) {
       let keys = [];
       if (event == 'modify') {
-        keys = ids.map(id => {
+        keys = ids.map((id) => {
           var { libraryID, key } = objectsClass.getLibraryAndKeyFromID(id);
           return libraryID == this.libraryID ? key : false;
         });
       } else
       if (event == 'delete') {
-        keys = ids.map(id => {
+        keys = ids.map((id) => {
           if (!extraData[id]) return false;
           var { libraryID, key } = extraData[id];
           return libraryID == this.libraryID ? key : false;
         });
       }
-      keys.filter(key => key).forEach(key => changedObjects.add(key));
+      keys.filter((key) => key).forEach((key) => changedObjects.add(key));
     }.bind(this) },
 
   [objectType],
@@ -1206,7 +1239,7 @@ Zotero.Sync.Data.Engine.prototype._uploadObjects = Zotero.Promise.coroutine(func
       // Remove selected and skipped objects from queue
       queue.splice(0, batch.length + numSkipped);
 
-      let jsonBatch = batch.map(o => o.json);
+      let jsonBatch = batch.map((o) => o.json);
 
       Zotero.debug("UPLOAD BATCH:");
       Zotero.debug(jsonBatch);
@@ -1264,23 +1297,23 @@ Zotero.Sync.Data.Engine.prototype._uploadObjects = Zotero.Promise.coroutine(func
           }
           // Unchanged
           else {
-              // If a cache object exists, there shouldn't be any need to update it, since
-              // nothing was changed on the server, including the version.
-              //
-              // If there's no cache object, we can save the uploaded object, which we know
-              // matches what's on the server. The version number won't necessarily reflect
-              // the actual version of the object on the server, since objects are uploaded
-              // in batches and we only get the final version, but it will guarantee that
-              // the object won't be redownloaded unnecessarily in the case of a full sync,
-              // because the version will be higher than whatever version is on the server.
-              let hasCacheObject = !!Zotero.Sync.Data.Local.getCacheObject(
-              objectType, obj.libraryID, obj.key, obj.version);
+            // If a cache object exists, there shouldn't be any need to update it, since
+            // nothing was changed on the server, including the version.
+            //
+            // If there's no cache object, we can save the uploaded object, which we know
+            // matches what's on the server. The version number won't necessarily reflect
+            // the actual version of the object on the server, since objects are uploaded
+            // in batches and we only get the final version, but it will guarantee that
+            // the object won't be redownloaded unnecessarily in the case of a full sync,
+            // because the version will be higher than whatever version is on the server.
+            let hasCacheObject = !!Zotero.Sync.Data.Local.getCacheObject(
+            objectType, obj.libraryID, obj.key, obj.version);
 
-              if (!hasCacheObject) {
-                jsonBatch[index].version = libraryVersion;
-                toCache.push(jsonBatch[index]);
-              }
+            if (!hasCacheObject) {
+              jsonBatch[index].version = libraryVersion;
+              toCache.push(jsonBatch[index]);
             }
+          }
 
           numSuccessful++;
           // Remove from batch to mark as successful
@@ -1352,19 +1385,19 @@ Zotero.Sync.Data.Engine.prototype._uploadObjects = Zotero.Promise.coroutine(func
         }
         // 500 errors should stay in queue and be retried, unless a dependency also failed
         else if (objectType == 'item') {
-            // Check parent item
-            let parentItem = batch[index].json.parentItem;
-            if (parentItem) {
-              for (let i in batch) {
-                if (i == index) break;
-                let o = batch[i];
-                if (o.failed && o.json.key) {
-                  Zotero.debug(`Not retrying child of failed parent ${parentItem}`);
-                  batch[index].failed = true;
-                }
+          // Check parent item
+          let parentItem = batch[index].json.parentItem;
+          if (parentItem) {
+            for (let i in batch) {
+              if (i == index) break;
+              let o = batch[i];
+              if (o.failed && o.json.key) {
+                Zotero.debug(`Not retrying child of failed parent ${parentItem}`);
+                batch[index].failed = true;
               }
             }
           }
+        }
       }
 
       // Add failed objects back to end of queue
@@ -1428,6 +1461,58 @@ Zotero.Sync.Data.Engine.prototype._uploadDeletions = Zotero.Promise.coroutine(fu
 });
 
 
+/**
+ * Update createdByUserID/lastModifiedByUserID for previously downloaded group items
+ *
+ * TEMP: Currently only processes one batch of items, but before we start displaying the names,
+ * we'll need to update it to fetch all
+ */
+Zotero.Sync.Data.Engine.prototype._updateGroupItemUsers = async function () {
+  // TODO: Do more at once when we actually start showing these names
+  var max = this.apiClient.MAX_OBJECTS_PER_REQUEST;
+
+  var sql = "SELECT key FROM items LEFT JOIN groupItems GI USING (itemID) " +
+  `WHERE libraryID=? AND GI.itemID IS NULL ORDER BY itemID LIMIT ${max}`;
+  var keys = await Zotero.DB.columnQueryAsync(sql, this.libraryID);
+  if (!keys.length) {
+    return;
+  }
+
+  Zotero.debug(`Updating item users in ${this.library.name}`);
+
+  var { json: jsonItems, error } = await this.apiClient.downloadObjects(
+  this.library.libraryType, this.libraryTypeID, 'item', keys)[
+  0];
+
+  if (error) {
+    Zotero.logError(error);
+    return;
+  }
+
+  for (let jsonItem of jsonItems) {
+    let item = Zotero.Items.getByLibraryAndKey(this.libraryID, jsonItem.key);
+    let params = [null, null];
+
+    // This should almost always exist, but maybe doesn't for some old items?
+    if (jsonItem.meta.createdByUser) {
+      let { id: userID, username, name } = jsonItem.meta.createdByUser;
+      await Zotero.Users.setName(userID, name !== '' ? name : username);
+      params[0] = userID;
+    }
+
+    if (jsonItem.meta.lastModifiedByUser) {
+      let { id: userID, username, name } = jsonItem.meta.lastModifiedByUser;
+      await Zotero.Users.setName(userID, name !== '' ? name : username);
+      params[1] = userID;
+    }
+
+    await item.updateCreatedByUser.apply(item, params);
+  }
+
+  return;
+};
+
+
 Zotero.Sync.Data.Engine.prototype._getJSONForObject = function (objectType, id, options = {}) {
   return Zotero.DB.executeTransaction(function* () {
     var objectsClass = Zotero.DataObjectUtilities.getObjectsClassForObjectType(objectType);
@@ -1448,13 +1533,13 @@ Zotero.Sync.Data.Engine.prototype._getJSONForObject = function (objectType, id,
     }
     // If copy of object in cache, use patch mode with cache data as the base
     else if (cacheObj) {
-        var mode = 'patch';
-        patchBase = cacheObj.data;
-      }
-      // Otherwise use full mode if there's a version
-      else {
-          var mode = obj.version ? "full" : "new";
-        }
+      var mode = 'patch';
+      patchBase = cacheObj.data;
+    }
+    // Otherwise use full mode if there's a version
+    else {
+      var mode = obj.version ? "full" : "new";
+    }
     return obj.toJSON({
       mode,
       includeKey: true,
@@ -1471,21 +1556,21 @@ Zotero.Sync.Data.Engine.prototype._getJSONForObject = function (objectType, id,
 
 
 /**
-    * Upgrade library to current sync architecture
-    *
-    * This sets the 'synced' and 'version' properties based on classic last-sync times and object
-    * modification times. Objects are marked as:
-    *
-    * - synced=1 if modified locally before the last classic sync time
-    * - synced=0 (unchanged) if modified locally since the last classic sync time
-    * - version=<remote version> if modified remotely before the last classic sync time
-    * - version=0 if modified remotely since the last classic sync time
-    *
-    * If both are 0, that's a conflict.
-    *
-    * @return {Object[]} - Objects returned from getVersions(), keyed by objectType, for use
-    *                      by _fullSync()
-    */
+ * Upgrade library to current sync architecture
+ *
+ * This sets the 'synced' and 'version' properties based on classic last-sync times and object
+ * modification times. Objects are marked as:
+ *
+ * - synced=1 if modified locally before the last classic sync time
+ * - synced=0 (unchanged) if modified locally since the last classic sync time
+ * - version=<remote version> if modified remotely before the last classic sync time
+ * - version=0 if modified remotely since the last classic sync time
+ *
+ * If both are 0, that's a conflict.
+ *
+ * @return {Object[]} - Objects returned from getVersions(), keyed by objectType, for use
+ *                      by _fullSync()
+ */
 Zotero.Sync.Data.Engine.prototype._upgradeCheck = Zotero.Promise.coroutine(function* () {
   var libraryVersion = this.library.libraryVersion;
   if (libraryVersion) return;
@@ -1630,18 +1715,18 @@ Zotero.Sync.Data.Engine.prototype._upgradeCheck = Zotero.Promise.coroutine(funct
 
 
 /**
-     * Perform a full sync
-     *
-     * Get all object versions from the API and compare to the local database. If any objects are
-     * missing or outdated, download them. If any local objects are marked as synced but aren't available
-     * remotely, mark them as unsynced for later uploading.
-     *
-     * (Technically this isn't a full sync on its own, because local objects are only flagged for later
-     * upload.)
-     *
-     * @param {Object[]} [versionResults] - Objects returned from getVersions(), keyed by objectType
-     * @return {Promise<Integer>} - Promise for the library version after syncing
-     */
+ * Perform a full sync
+ *
+ * Get all object versions from the API and compare to the local database. If any objects are
+ * missing or outdated, download them. If any local objects are marked as synced but aren't available
+ * remotely, mark them as unsynced for later uploading.
+ *
+ * (Technically this isn't a full sync on its own, because local objects are only flagged for later
+ * upload.)
+ *
+ * @param {Object[]} [versionResults] - Objects returned from getVersions(), keyed by objectType
+ * @return {Promise<Integer>} - Promise for the library version after syncing
+ */
 Zotero.Sync.Data.Engine.prototype._fullSync = Zotero.Promise.coroutine(function* (versionResults) {
   Zotero.debug("Performing a full sync of " + this.library.name);
 
@@ -1684,12 +1769,12 @@ Zotero.Sync.Data.Engine.prototype._fullSync = Zotero.Promise.coroutine(function*
       }
       // If not available, get from API
       else {
-          results = yield this.apiClient.getVersions(
-          this.library.libraryType,
-          this.libraryTypeID,
-          objectType);
+        results = yield this.apiClient.getVersions(
+        this.library.libraryType,
+        this.libraryTypeID,
+        objectType);
 
-        }
+      }
       if (lastLibraryVersion != results.libraryVersion) {
         yield this._onLibraryVersionChange();
         continue loop;
@@ -1735,9 +1820,9 @@ Zotero.Sync.Data.Engine.prototype._fullSync = Zotero.Promise.coroutine(function*
         Zotero.debug(remoteMissing);
 
         let toUpload = remoteMissing.map(
-        key => objectsClass.getIDFromLibraryAndKey(this.libraryID, key)
+        (key) => objectsClass.getIDFromLibraryAndKey(this.libraryID, key)
         // Remove any objects deleted since getAllKeys() call
-        ).filter(id => id);
+        ).filter((id) => id);
 
         // For remotely missing objects that exist locally, reset version, since old
         // version will no longer match remote, and mark for upload
@@ -1812,7 +1897,7 @@ Zotero.Sync.Data.Engine.prototype._restoreToServer = async function () {
 
     // Filter to objects that don't exist locally and delete those objects remotely
     let remoteKeys = Object.keys(results.versions);
-    let locallyMissingKeys = remoteKeys.filter(key => {
+    let locallyMissingKeys = remoteKeys.filter((key) => {
       return !objectsClass.getIDFromLibraryAndKey(this.libraryID, key);
     });
     if (locallyMissingKeys.length) {
@@ -1890,7 +1975,7 @@ Zotero.Sync.Data.Engine.prototype._restoreToServer = async function () {
 
 Zotero.Sync.Data.Engine.prototype._getOptions = function (additionalOpts = {}) {
   var options = {};
-  this.optionNames.forEach(x => options[x] = this[x]);
+  this.optionNames.forEach((x) => options[x] = this[x]);
   for (let opt in additionalOpts) {
     options[opt] = additionalOpts[opt];
   }
@@ -1974,9 +2059,9 @@ Zotero.Sync.Data.Engine.prototype._checkObjectUploadError = Zotero.Promise.corou
 
           let id = parentItem.id;
           // If parent item isn't already in queue, mark it as unsynced and add it
-          if (!queue.find(o => o.id == id)
+          if (!queue.find((o) => o.id == id)
           // TODO: Don't use 'delete' on batch, which results in undefineds
-          && !batch.find(o => o && o.id == id)) {
+          && !batch.find((o) => o && o.id == id)) {
             yield Zotero.Sync.Data.Local.markObjectAsUnsynced(parentItem);
             Zotero.logError(`Adding parent item ${data.parentItem} to upload queue`);
             queue.push({
@@ -1991,6 +2076,12 @@ Zotero.Sync.Data.Engine.prototype._checkObjectUploadError = Zotero.Promise.corou
         }
       }
     }
+
+    // Split old annotations before we were splitting them automatically in the PDF reader
+    if (message == "Annotation position is too long") {
+      let item = Zotero.Items.getByLibraryAndKey(this.libraryID, key);
+      yield Zotero.Annotations.splitAnnotations(item);
+    }
   } else
   if (code == 403) {
     // If we get a 403 for a local group attachment, check the group permissions to confirm
@@ -2038,8 +2129,8 @@ Zotero.Sync.Data.Engine.prototype._checkObjectUploadError = Zotero.Promise.corou
   // prevent an outdated upload before the object version is checked. If it does, we need to
   // do a full sync. This error is checked in handleUploadError().
   else if (code == 404 || code == 412) {
-      throw e;
-    }
+    throw e;
+  }
 
   return false;
 });
